module SALTBase

using LinearAlgebra
using CatViews
using DualNumbers  # used in gain.jl
using AbbreviatedTypes
using Printf: @printf

export AbstractMaxwellSolver

# Each concrete SALT solver package (e.g., MaxwellSALT) should define a concrete subtype of
# AbstractMaxwellSolver to store the information (e.g., the Maxwell operator A) needed by the
# specific linear Maxwell solver to use in the SALT solver package.
abstract type AbstractMaxwellSolver end

# Each concrete SALT solver package (e.g., MaxwellSALT) should extend the following Base
# functions:
#
# - similar(mxsolver:AbstractMaxwellSolver)
# - size(mxsolver::AbstractMaxwellSolver)

# Each concrete SALT solver package (e.g., MaxwellSALT) should implement a concrete subtype of
# AbstractMaxwellSolver and extend the following functions to be used in lasing.jl and nonlasing.jl:
#
# init_mxsolver!(mxsolver::AbstractMaxwellSolver, ω::Number, ε::AbsVecNumber)
# mxsolve!(x::AbsVecComplexF, mxsolver::AbstractMaxwellSolver, b::AbsVecNumber)
# mxsolve_transpose!(x::AbsVecComplexF, mxsolver::AbstractMaxwellSolver, b::AbsVecNumber)
# mxapply!(b::AbsVecComplexF, mxsolver::AbstractMaxwellSolver, x::AbsVecNumber)
# abs2_interp!(abs2ψ::AbsVecReal, mxsolver::AbstractMaxwellSolver, ψ::AbsVecNumber)
function init_mxsolver! end  # initialize mxsolver for current ω and ε
function mxsolve! end  # x = A⁻¹ b
function mxsolve_transpose! end  # x = A⁻ᵀ b
function mxapply! end  # b = A x
function abs2_interp! end  # abs2ψ = abs2.(interpolate(ψ))

include("util.jl")
include("gain.jl")
include("lasing.jl")
include("nonlasing.jl")
include("anderson.jl")
include("switching.jl")
include("simulation.jl")

end # module
